#include <Wire.h>
#include "rgb_lcd.h"
#include <cstdlib>
#include <cstring>

rgb_lcd lcd;

const int white[] = {255, 255, 255};
const int red[] = {255, 0, 0};

const int pinRot = A0;
const int pinTemp = A1;
const int pinIR = A2;
const int pinUV = A3;

const int pinBut = 7;
const int pinBuz = 8;

const int tempref = 3975;

int statecount = 0;
int tanperc = 0;
int tantarg = 0;
float alltemp[8] = {0, 0, 0, 0, 0, 0, 0, 0};
//float allIR[8] = {0, 0, 0, 0, 0, 0, 0, 0};
float alluv[8] = {0, 0, 0, 0, 0, 0, 0, 0};
int pt = 0;
float acctemp = 0;
//float accIR = 0;
float accUV = 0;
float exposure = 0;
const int displayrate = 40;
const int burnrate = 170;

int maxuv = 0;
int maxtemp = 0;
unsigned long starttime = 0;
unsigned long endtime = 0;

void setup() {
  Serial.begin(9600);
  lcd.begin(16, 2);
  lcd.setRGB(white[0], white[1], white[2]);
  lcd.setCursor(0, 0);
  lcd.print("Hello, I'm your");
  lcd.setCursor(0, 1);
  lcd.print("Solmate :)");
  pinMode(pinBut, INPUT);
  digitalWrite(pinBut, HIGH);
  pinMode(pinBuz, OUTPUT);
  statecount = 0;
  delay(4000);
  setTan();
}

void clrRow(int row) {
  lcd.setCursor(0, row);
  lcd.print("                ");
}

void setTan() {
  lcd.setRGB(white[0], white[1], white[2]);
  lcd.setCursor(0, 0);
  lcd.print("Set your tan %! ");
  clrRow(1);
  while (!digitalRead(pinBut)) {
    int rotread = analogRead(pinRot);
    tanperc = rotread * 100 / 1015;
    tantarg = rotread * burnrate / 1015;
    char perc[] = "100%";
    int targ = tanperc;
    for (int i = 2; i >= 0; i--) {
      if (i < 2 && targ == 0) {
        perc[i] = ' ';
      } else {
        perc[i] = '0' + targ%10;
      }
      targ /= 10;
    }
    lcd.setCursor(6, 1);
    lcd.print(perc);
    delay(100);
  }
  exposure = 0;
  while (digitalRead(pinBut));
  lcd.clear();
  starttime = millis();
  maxuv = 0;
  maxtemp = 0;
}

boolean readFile(){
  //Get the query result from file
  char data[16];
  FILE *fp;
  fp = fopen("/tmp/tantweet.txt", "r");
  if(fp!=NULL){
    fgets(data, 16, fp);
    fclose(fp);
    if (data[0] == '1') return true;
    else return false;
  }else{
    return false;
  }
}

void tanned() {
  endtime = millis();
  lcd.setRGB(red[0], red[1], red[2]);
  clrRow(0);
  clrRow(1);
  lcd.setCursor(0, 0);
  lcd.print("Perfectly tanned!");
  digitalWrite(pinBuz, HIGH);
  delay(500);
  digitalWrite(pinBuz, LOW);
  delay(500);
  digitalWrite(pinBuz, HIGH);
  delay(500);
  digitalWrite(pinBuz, LOW);
  delay(500);
  digitalWrite(pinBuz, HIGH);
  delay(500);
  digitalWrite(pinBuz, LOW);
  delay(500);
  lcd.setRGB(white[0], white[1], white[2]);
  clrRow(0);
  lcd.setCursor(0, 0);
  lcd.print("Tweet it?");
  int tweet = 0;
  while (!digitalRead(pinBut)) {
    //select yes or no
    int rotread = analogRead(pinRot);
    lcd.setCursor(0, 1);
    if (rotread < 512) {
      lcd.print("   >Yes    No   ");
      tweet = 1;
    } else {
      lcd.print("    Yes   >No   ");
      tweet = 0;
    }
    delay(100);
  }
  while (digitalRead(pinBut));
  if (tweet == 1) {
    String cmd = "python /home/root/test7.py -m ";
    cmd = cmd + tanperc + " -e ";
    cmd = cmd + ((endtime-starttime)/60000) + " -t ";
    cmd = cmd + maxtemp + " -u ";
    cmd = cmd + maxuv + " > /tmp/tantweet.txt";
    //Excute the query, and get the result
    system(cmd.buffer);
    lcd.clear();
    if (readFile()) {
      lcd.setRGB(white[0], white[1], white[2]);
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Tweeted!");
    } else {
      lcd.setRGB(white[0], white[1], white[2]);
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Failed to");
      lcd.setCursor(0, 1);
      lcd.print("tweet...");
    }
    delay(4000);
  }
}

void loop() {
  
  Serial.println("hello");
  
  //check if finished
  if (exposure >= tantarg) {
    tanned();
    setTan();
  } else {
    //check if reset
    if (digitalRead(pinBut)) {
      while (digitalRead(pinBut));
      setTan();
    }
  }

  //update all sensor inputs
  
  //temperature
  int tempread = analogRead(pinTemp);
  float res = (float)(1023-tempread)*10000/tempread;
  float temp = 1/(log(res/10000)/tempref+1/298.15)-273.15;
  temp = temp*1.8 + 32;
  acctemp += temp - alltemp[pt];
  alltemp[pt] = temp;
  Serial.print("temp:");
  Serial.println(temp);
  
  //UV
  float uv = analogRead(pinUV);
  uv = uv * 4980.0/1023.0 * 0.307 / 200.0;
  accUV += uv - alluv[pt];
  alluv[pt] = uv;
  Serial.print("uv:");
  Serial.println(accUV/8);
  
  pt = (pt < 7? pt+1 : 0);
  
  //output data every  0.5 sec
  if (statecount % 10 == 0) {
    if (acctemp/8 > maxtemp) {
      maxtemp = (acctemp/8);
    }
    if (accUV/8 > maxuv) {
      maxuv = (accUV/8);
    }
    if (statecount < displayrate ) {
      //temperature
      char buffer[16];
      sprintf(buffer, "Temp %0.1f F", acctemp/8);
      clrRow(0);
      lcd.setCursor(2, 0);
      lcd.print(buffer);
    } else {
      //UV
      char buffer[16];
      sprintf(buffer, "UV Index %0.1f", accUV/8);
      clrRow(0);
      lcd.setCursor(2, 0);
      lcd.print(buffer);
    }
    
    //update tan
    lcd.setCursor(0, 1);
    for (int i = 0; i < 16 * exposure / tantarg; i++) {
      lcd.print("X");
    }
  }
  
  exposure += uv/600;
  
  
  statecount = (statecount < 2*displayrate? statecount+1 : 0);
  delay(100);
}
